//data used is infrastructure distribution of power
const data = {
  name: 'Energy Infrastructure',
  children: [
    {
      name: 'Power Plants',
      value: 2,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Coal Power Plant',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 100,
        },
        {
          name: 'Solar Power Plant',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 90,
        },
      ],
    },
    {
      name: 'Substations',
      value: 3,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Transmission Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 150,
        },
        {
          name: 'Distribution Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 230,
        },
        {
          name: 'Industrial Zone Substation',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 95,
        },
      ],
    },
    {
      name: 'Distribution Centers',
      value: 3,
      type: 'blue',
      level: 'green',
      children: [
        {
          name: 'Urban Area Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 80,
        },
        {
          name: 'Rural Area Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 170,
        },
        {
          name: 'Commercial District Distribution Center',
          value: 1,
          type: 'red',
          level: '#863a9f',
          capacity: 205,
        },
      ],
    },
  ],
};

const margin = {
  top: 20,
  right: 320,
  bottom: 200,
  left: 200,
};
const width = 850 - margin.left - margin.right;
const height = 850 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

let nodes = d3.hierarchy(data, (node) => node.children);
nodes = tree(nodes);

const svgExists = d3.select('svg').empty();
//for background and main svg creation

if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom)
    .style('background-color', 'lightblue');

  const g = svg
    .append('g')
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );
  //creation of links 

  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => d.data.level)
    .attr('d', (d) => {
      return (
        'M' +
        d.y +
        ',' +
        d.x +
        'C' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.x +
        ' ' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.parent.x +
        ' ' +
        d.parent.y +
        ',' +
        d.parent.x
      );
    });
//node creation
  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  node
    .append('circle')
    .attr('r', (d) => d.data.value)
    .style('stroke', (d) => d.data.type)
    .style('fill', (d) => d.data.level);

  node
    .append('text')
    .attr('dy', '.100em')
    .attr('x', (d) =>
      d.children
        ? (d.data.value + 5) * -1
        : d.data.value + 5,
    )
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -(d.data.value + 5) : d,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => {
      if (d.data.capacity !== undefined) {
        return `${d.data.name} (${d.data.capacity})`;
      } else {
        return d.data.name;
      }
    });
}
